/* Copyright (c) 2013 Oberon microsystems, Inc. (Switzerland)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. */

// Originally developed for the book
//   "Getting Started with the Internet of Things", by Cuno Pfister.
//   Copyright 2011 Cuno Pfister, Inc., 978-1-4493-9357-1.
//
// Version 1.1, for the .NET Micro Framework release 4.2.

// Use for NETMF, comment out for NETFX.
using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Net.NetworkInformation;

namespace Gsiot.Http.Sockets
{
    internal static class NetworkInterfaceConfiguration
    {
        static NetworkInterface[] nis = null;
        static string physicalAddress = "NetworkInterface: could not determine MAC address";
        static bool isDhcpEnabled = false;

        static void ReadConfiguration()
        {
            nis = NetworkInterface.GetAllNetworkInterfaces();
            if ((nis != null) && (nis.Length == 1))
            {
                NetworkInterface ni = nis[0];
                byte[] adr = ni.PhysicalAddress;
                if ((adr[0] == 0x00) &&
                    (adr[1] == 0x50) &&
                    (adr[2] == 0xc2) &&
                    (adr[3] == 0xf8) &&
                    (adr[4] == 0x00) &&
                    (adr[5] == 0x00))
                {
                    throw new Exception("Please set the MAC address of your Mountaineer mainboard to the address printed on the Ethernet connector. You can use the tool MFDeploy for this purpose.");
                }
                string s = adr[0].ToString("X");
                s = s + "-" + adr[1].ToString("X");
                s = s + "-" + adr[2].ToString("X");
                s = s + "-" + adr[3].ToString("X");
                s = s + "-" + adr[4].ToString("X");
                s = s + "-" + adr[5].ToString("X");
                physicalAddress = s;
                Debug.Print("MAC address is " + physicalAddress);
                isDhcpEnabled = ni.IsDhcpEnabled;
                if (isDhcpEnabled)
                {
                    Debug.Print("DHCP is enabled");
                }
                else
                {
                    Debug.Print("DHCP is disabled (WARNING: make sure that you have set a unique static IP address. You can use the MFDeploy tool for this purpose.)");
                }
            }
        }

        static internal string PhysicalAddress()
        {
            if (nis == null) { ReadConfiguration(); }
            return physicalAddress;
        }

        static internal bool IsDhcpEnabled()
        {
            if (nis == null) { ReadConfiguration(); }
            return isDhcpEnabled;
        }
    }
}
