// http://msdn.microsoft.com/en-us/library/dd264808.aspx

namespace System.Diagnostics.Contracts
{
    public static class Contract
    {
        /// <summary>
        /// Throws an exception if a precondition is violated.
        /// </summary>
        /// <param name="condition">Precondition to be checked.</param>
        public static void Requires(bool condition)
        {
            if (!condition)
            {
                throw new Exception("precondition failed");
            }
        }

        /// <summary>
        /// Throws an exception if a postcondition is violated.
        /// </summary>
        /// <param name="condition">Postcondition to be checked.</param>
        public static void Ensures(bool condition)
        {
            if (!condition)
            {
                throw new Exception("postcondition failed");
            }
        }

        /// <summary>
        /// Throws an exception if an invariant is violated,
        /// e.g., a loop invariant or object invariant.
        /// </summary>
        /// <param name="condition">Condition to be checked, e.g.,
        /// loop or object invariant.</param>
        public static void Invariant(bool condition)
        {
            if (!condition) { throw new Exception("invariant failed"); }
        }

        /// <summary>
        /// Throws an exception if some other condition is violated.
        /// </summary>
        /// <param name="condition">Condition to be checked.</param>
        public static void Assert(bool condition)
        {
            if (!condition) { throw new Exception("assertion violated"); }
        }
    }
}
