﻿using System.Diagnostics.Contracts;

namespace System.Threading
{
    public delegate void WaitCallback(object state);

    internal class WorkItem
    {
        internal readonly WaitCallback callback;
        internal readonly object state;

        internal WorkItem(WaitCallback callback, object state)
        {
            Contract.Requires(callback != null);
            this.callback = callback;
            this.state = state;
        }
    }

    public static class ThreadPool
    {
        static Thread[] threads;
        static BoundedQueue workItemQueue;

        public static void Open(int nofThreads)
        {
            Contract.Requires(nofThreads > 0);
            threads = new Thread[nofThreads];
            var i = 0;
            while (i != nofThreads)
            {
                threads[i] = new Thread(Run);
                threads[i].Priority = ThreadPriority.Lowest;
                i = i + 1;
            }
            workItemQueue = new BoundedQueue { Capacity = nofThreads };
        }

        public static void Start()
        {
            var i = 0;
            while (i != threads.Length)
            {
                threads[i].Start();
                i = i + 1;
            }
        }

        public static bool QueueUserWorkItem(WaitCallback callback,
                                                object state)
        {
            Contract.Requires(callback != null);
            Contract.Requires(threads != null);
            var item = new WorkItem(callback, state);
            workItemQueue.Enqueue(item);
            return true;
        }

        static void Run()
        {
            while (true)
            {
                var obj = workItemQueue.Dequeue();      // may block!
                var item = (WorkItem)obj;
                try
                {
                    item.callback(item.state);
                }
                catch (Exception)
                {
                    //Debug.Print("ThreadPool.Run exception in callback:\r\n" + e);
                }
            }
        }
    }
}
